@@ -11,9 +11,10 @@ use super::{
 };
 
 pub struct VkDevice {
-    pub device: Arc<RawDevice>,
     pub physical_device: vk::PhysicalDevice,
     pub memory_properties: vk::PhysicalDeviceMemoryProperties,
+    pub device: Arc<RawDevice>,
+    pub instance: Arc<VkInstance>,
 }
 
 pub struct RawDevice {
@@ -59,14 +60,6 @@ impl std::ops::Deref for VkDevice {
     }
 }
 
-// Do not do this, you lack!
-//
-// impl std::ops::DerefMut for VkDevice {
-//     fn deref_mut(&mut self) -> &mut Self::Target {
-//         Arc::get_mut(&mut self.device)
-//     }
-// }
-
 impl VkDevice {
     pub fn get_device_properties(&self, instance: &VkInstance) -> VkDeviceProperties {
         let (properties, features, memory) = unsafe {
@@ -199,7 +192,7 @@ impl VkDevice {
             .get_present_modes(self)?
             .iter()
             .cloned()
-            .find(|&mode| mode == vk::PresentModeKHR::FIFO)
+            .find(|&mode| mode == vk::PresentModeKHR::IMMEDIATE)
             .unwrap_or(vk::PresentModeKHR::FIFO);
 
         let surface_format = {
@@ -471,24 +464,33 @@ impl VkDevice {
         &self,
         command_buffer: vk::CommandBuffer,
         src_image: vk::Image,
+        src_image_layout: vk::ImageLayout,
         dst_image: vk::Image,
-        src_extent: vk::Extent3D,
-        dst_extent: vk::Extent3D,
+        dst_image_layout: vk::ImageLayout,
+        extent: vk::Extent3D,
     ) {
-        let src_offset = [
+        self.set_image_layout(
+            command_buffer,
+            src_image,
+            src_image_layout,
+            vk::ImageLayout::TRANSFER_SRC_OPTIMAL,
+            vk::PipelineStageFlags::TOP_OF_PIPE,
+            vk::PipelineStageFlags::TRANSFER,
+        );
+        self.set_image_layout(
+            command_buffer,
+            dst_image,
+            dst_image_layout,
+            vk::ImageLayout::TRANSFER_DST_OPTIMAL,
+            vk::PipelineStageFlags::TOP_OF_PIPE,
+            vk::PipelineStageFlags::TRANSFER,
+        );
+        let offset = [
             vk::Offset3D { x: 0, y: 0, z: 0 },
             vk::Offset3D {
-                x: src_extent.width as i32,
-                y: src_extent.height as i32,
-                z: src_extent.depth as i32,
-            },
-        ];
-        let dst_offset = [
-            vk::Offset3D { x: 0, y: 0, z: 0 },
-            vk::Offset3D {
-                x: dst_extent.width as i32,
-                y: dst_extent.height as i32,
-                z: dst_extent.depth as i32,
+                x: extent.width as i32,
+                y: extent.height as i32,
+                z: extent.depth as i32,
             },
         ];
         let blit_region = [vk::ImageBlit::builder()
@@ -504,8 +506,8 @@ impl VkDevice {
                 layer_count: 1,
                 mip_level: 0,
             })
-            .src_offsets(src_offset)
-            .dst_offsets(dst_offset)
+            .src_offsets(offset)
+            .dst_offsets(offset)
             .build()];
 
         unsafe {
@@ -519,6 +521,22 @@ impl VkDevice {
                 vk::Filter::NEAREST,
             )
         };
+        self.set_image_layout(
+            command_buffer,
+            dst_image,
+            vk::ImageLayout::TRANSFER_DST_OPTIMAL,
+            dst_image_layout,
+            vk::PipelineStageFlags::TRANSFER,
+            vk::PipelineStageFlags::ALL_GRAPHICS,
+        );
+        self.set_image_layout(
+            command_buffer,
+            src_image,
+            vk::ImageLayout::TRANSFER_SRC_OPTIMAL,
+            src_image_layout,
+            vk::PipelineStageFlags::TRANSFER,
+            vk::PipelineStageFlags::ALL_GRAPHICS,
+        );
     }
 
     pub fn create_vk_buffer_from_slice<T>(
@@ -556,7 +574,7 @@ impl VkDevice {
         let memory_reqs = unsafe { self.get_buffer_memory_requirements(buffer) };
 
         let memory_type_index =
-            utils::find_memorytype_index(&memory_reqs, &self.memory_properties, memory_prop_flags)
+            utils::find_memory_type_index(&memory_reqs, &self.memory_properties, memory_prop_flags)
                 .unwrap();
         let mut mem_alloc_flags = vk::MemoryAllocateFlagsInfoKHR::default();
         let alloc_info = vk::MemoryAllocateInfo::builder()
@@ -586,6 +604,19 @@ impl VkDevice {
             mapped: None,
         })
     }
+
+    pub fn name_semaphore(&self, object: vk::Semaphore, name: &str) -> VkResult<()> {
+        self.instance
+            .name_object(self, object, vk::ObjectType::SEMAPHORE, name)
+    }
+    pub fn name_image(&self, object: vk::Image, name: &str) -> VkResult<()> {
+        self.instance
+            .name_object(self, object, vk::ObjectType::IMAGE, name)
+    }
+    pub fn name_queue(&self, object: vk::Queue, name: &str) -> VkResult<()> {
+        self.instance
+            .name_object(self, object, vk::ObjectType::QUEUE, name)
+    }
 }
 
 pub struct VkBuffer<'a, T> {
